use manifest::{self, Manifest};
use serde::{Deserialize, Serialize};
use std::{collections::HashMap, str::FromStr};
use toml;

#[derive(Serialize, Deserialize)]
pub struct PKGR {
    pub bootstrap: Option<Bootstrap>,
}

#[derive(Serialize, Deserialize)]
pub struct Bootstrap {
    check_installed_commands: Vec<String>,
    commands: Vec<String>
}

#[derive(Serialize, Deserialize)]
pub struct BadManifest {
    pkgr: Option<PKGR>,
}

pub fn mani_from_str(s: &str) -> Manifest<Option<PKGR>> {
    let mani = Manifest::from_str(s).unwrap();
    let bmani = toml::from_str::<BadManifest>(s).unwrap();
    Manifest::<Option<PKGR>> {
        package: mani.package,
        dependencies: mani.dependencies,
        fs: mani.fs,
        bin: mani.bin,
        build: mani.build,
        pkgr: bmani.pkgr
    }
}
